pub mod models;

pub use models::*;

pub enum AuthErr {
    UnknownSession,
    UnknownUser,
    InvalidPassword,
    Other(Box<dyn std::error::Error + Sync + Send>),
}

pub trait AuthRepository: Send + Sync {
    /// Validate the given session ID and return its user.
    fn validate_session(&self, session_id: i64) -> Result<models::User, AuthErr>;

    /// Create a new session for the given user.
    fn create_session(
        &self,
        username: &str,
        password: &str,
    ) -> Result<(i64, models::User), AuthErr>;
}

pub trait DeviceRepository: Send + Sync {
    /// Return all devices associated with the user
    fn devices_for_user(&self, user: &User) -> Result<Vec<Device>, AuthErr>;

    /// Update the information for the given device. If the device doesn't exist yet, it should be
    /// created.
    fn update_device_info(
        &self,
        user: &User,
        device: &str,
        patch: DevicePatch,
    ) -> Result<(), AuthErr>;
}
